﻿//------------------------------------------------------------------------------------------------------------------------------------------
//                              SDR ASM for Windows Ver 1.10    
//                              (C) Radiun.net T.Kato 2019
//
//                              Ver 1.00    2019/02/02      初版リリース
//                              Ver 1.01    2019/02/12      クロック設定がFPGAに送られていないバグを修正
//                              Ver 1.02    2019/02/13      クロック設定がFPGAに正常に送られていないバグを修正
//                              Ver 1.10    2019/02/27      WaveCapture機能追加
//------------------------------------------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO;                                        // File IO
using FTD2XX_NET;                                       // FTD2XX Wrapper DLL for C#

namespace piradio_w
{

    public partial class Form1 : Form
    {
        Bitmap DrawArea;

        //-------------------------------------------------------------------- グローバル変数クラスの定義 ----------------------------------
        public static class Global
        {
            public static double[] ave = new double[1024];

            public static ushort lastch1;
            public static ushort lastch2;

            public static int mousesw;
            public static int sbsw = 0;

            public static int fblk;                             // Focus Block
            public static ushort[] bv = new ushort[60];         // Block View
            public static int[] blkx = new int[60];             // Block Locate X
            public static int[] blky = new int[60];             // Block Locate Y
            public static int[] param1 = new int[60];           // Parameter Clock
            public static int[] param2 = new int[60];           // Parameter Freq / ADC No / CIC Shift / Const
            public static int[] param3 = new int[60];           // Parameter Phase
            public static int[] param4 = new int[60];           // Parameter Amplitude
            public static int[] param5 = new int[60];           // Parameter DC offset
            public static float[,] wave = new float[4096, 2];   // AWG Waveform Data
            public static double[] d1 = new double[1024];       // Wave data CH1
            public static double[] d2 = new double[1024];       // Wave data CH2
            public static double[] ff = new double[1024];       // Wave data FFT

        }

        //-------------------------------------------------------------------- フォームと初期値の定義 --------------------------------------
        public Form1()
        {
            InitializeComponent();

            //　タイマーインターバル 10ms
            timer1.Interval = 10;

            //　ブロック図の定義
            dimblock();

            //　ブロック全部を描画
            Block(0, 0);
        }

        //-------------------------------------------------------------------- FT232HLコマンド生成 -----------------------------------------
        private static byte[] FT232(ushort adrs)                                // FT232HL command generator
        {
            byte b1 = (byte)((adrs & 0xFF00) / 0x0100);                         // Command (bit 15-12) and Address (bit 11- 8)
            byte b2 = (byte)((adrs & 0x00FF) / 0x0001);                         // Address (bit  7- 0)

            byte[] buf = {  0x80, 0x00, 0x0B,                                   // ADBUS port command, CS -> L, port direction
                            0x86, 0x00, 0x00,                                   // Clock divider 60MHz/((1+0x00*256+0x00)*2)=30MHz
                            //0x86, 0x01, 0x00,                                   // Clock divider 60MHz/((1+0x00*256+0x01)*2)=15MHz
                            0x11, 0x01, 0x00,                                   // SPI port command TX, 2byte(0x01 0x00)
                            b1, b2,                                             // Command and Address 
                            0x20, 0x01, 0x00, 0x87,                             // SPI port command RX, 2byte(0x01 0x00), Send immediate
                            0x80, 0x08, 0x0B,                                   // ADBUS port command, CS -> H, port direction
                         };

            return buf;
        }

        //-------------------------------------------------------------------- タイマーイベント --------------------------------------------
        private void timer1_Tick(object sender, EventArgs e)
        {
            Capture();
        }

        //-------------------------------------------------------------------- メインルーチン ----------------------------------------------
        private void Capture()
        {
            int cnt = 0;
            float lv = 0;
            int[] x = new int[1024];
            int[] v1 = new int[1024];
            int[] v2 = new int[1024];
            int[] v3 = new int[1024];
            double[] d1 = new double[1024];
            double[] d2 = new double[1024];

            byte[] ch1 = new byte[2048];                                        // Substitution ch1[0-2048]
            byte[] ch2 = new byte[2048];                                        // Substitution ch2[0-2048]
            byte[] ch3 = new byte[2048];                                        // Substitution ch2[0-2048]
            byte[] wbuf;

            uint writtenLength = 0;
            uint readLength = 0;
            uint RxLength = 0;
            uint bx = 0;
            uint by = 0;
            ushort adrs = 0x0000;

            FTDI ftdev = new FTDI();                                            // Device Driver
            ftdev.OpenByIndex(0);                                               // Open Device Index=0
            ftdev.SetBitMode(0, 2);                                             // Mask=0, Mode=2(MPSSE)

            //　グラフィックス・オブジェクトの作成 1
            Graphics g1;

            int pw1 = pictureBox1.Size.Width;                                    // Get Picture Box size
            int ph1 = pictureBox1.Size.Height;
            int pws1 = (int)((float)pw1 / 15.625);
            float phs1 = (float)(ph1 / 16.0);
            float phh1 = (float)(ph1 / 2.0);
            float phb1 = (float)(phh1 - 8.0 * phs1);

            //　グラフィックス・オブジェクトの作成 2
            Graphics g2;

            int pw2 = pictureBox1.Size.Width;                                    // Get Picture Box size
            int ph2 = pictureBox1.Size.Height;
            int pws2 = (int)((float)pw2 / 15.625);
            int phs2 = ph2 / 10;
            int phh2 = ph2 / 2;

            //　スペアナ窓関数ラジオボタン
            int winsw = 0;
            if (radioButton5.Checked == true) winsw = 0;
            if (radioButton6.Checked == true) winsw = 1;
            if (radioButton7.Checked == true) winsw = 2;

            //　UP/DOWNスイッチの値を取得
            ushort span = (ushort)numericUpDown1.Value;
            ushort amp1 = (ushort)numericUpDown2.Value;
            ushort amp2 = (ushort)numericUpDown3.Value;
            int pos1 = (int)numericUpDown5.Value;
            int pos2 = (int)numericUpDown6.Value;
            ushort hrz = (ushort)numericUpDown7.Value;
            int sref = (int)numericUpDown8.Value;
            int sdiv = (int)numericUpDown9.Value;

            //　オシロの初期設定
            ushort clk = (ushort)Global.param1[20] ;                            // Oscillo Clock
            ushort CH1 = 6;
            ushort CH2 = 7;

            //　オシロの時間軸の表示
            if (Global.sbsw == 0)                                               // SDR Block HF
            {
                if (clk < 13)
                {
                    float dt = (float)(Math.Pow(2, clk - 1) / 128);
                    float st = (float)(dt * hrz);
                    string sst = st.ToString("F1") + "us";
                    textBox16.Text = sst;
                }
                else
                {
                    float dt = (float)(Math.Pow(2, clk - 1) / 128000);
                    float st = (float)(dt * hrz);
                    string sst = st.ToString("F3") + "ms";
                    textBox16.Text = sst;
                }
            }
            else                                                                // SDR Block AM
            {
                float dt = (float)(Math.Pow(2, clk - 1) / 1000);
                float st = (float)(dt * hrz);
                string sst = st.ToString("F3") + "ms";
                textBox16.Text = sst;
            }

            //　スペアナのスパンの表示
            if (Global.sbsw == 0)                                               // SDR Block HF
            {
                if (clk < 9)
                {
                    float fclk = (float)(128 / Math.Pow(2, clk - 1));
                    float sp = (float)(fclk * span / 512);
                    string ssp = sp.ToString("F1") + "MHz";
                    textBox17.Text = ssp;
                }
                else
                {
                    float fclk = (float)(128000 / Math.Pow(2, clk - 1));
                    float sp = (float)(fclk * span / 512);
                    string ssp = sp.ToString("F3") + "kHz";
                    textBox17.Text = ssp;
                }
            }
            else                                                                // SDR Block AM
            {
                float fclk = (float)(1000 / Math.Pow(2, clk - 1));
                float sp = (float)(fclk * span / 512);
                string ssp = sp.ToString("F1") + "kHz";
                textBox17.Text = ssp;
            }

            //　描画色の指定
            Pen bpen = new Pen(Color.Blue, 1);
            Pen rpen = new Pen(Color.Red, 1);
            Pen gpen = new Pen(Color.DarkGreen, 1);
            Pen apen = new Pen(Color.Aqua, 1);

            if (Global.lastch1 != CH1)
            {
                //-------------------------------------------------------------------- CH1 観測点の選択 --------------------------------------------
                bx = 15;                                                                    // Output CH1
                by = CH1;                                                                   // Input CIC1
                Global.lastch1 = CH1;                                                       // Memory Last Node of CH1                                                      
                adrs = (ushort)(0x4000 + bx * 32 + by);                                     // Circuit define + X + Y
                wbuf = FT232(adrs);                                                         // Get Command
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command
            }

            if (Global.lastch2 != CH2)
            {
                //-------------------------------------------------------------------- CH2 観測点の選択 --------------------------------------------
                bx = 16;                                                                    // Output CH2
                by = CH2;                                                                   // Input CIC2
                Global.lastch2 = CH2;                                                       // Memory Last Node of CH2
                adrs = (ushort)(0x4000 + bx * 32 + by);                                     // Circuit define + X + Y
                wbuf = FT232(adrs);                                                         // Get Command
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command
            }

            do
            {
                //-------------------------------------------------------------------- キャプチャ　リセット ----------------------------------------
                adrs = clk;                                                                 // RAM Single mode CLK=Select
                wbuf = FT232(adrs);                                                         // Get Command
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command

                //-------------------------------------------------------------------- キャプチャ　スタート ----------------------------------------
                wbuf = FT232(0x1000);                                                       // Capture Start
                ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                          // Write Command
                Thread.Sleep(5);                                                            // Wait Capture done (3ms)

                //-------------------------------------------------------------------- FTDI バッファを空にする -------------------------------------
                ftdev.GetRxBytesAvailable(ref RxLength);                                    // Get number of Garbage
                ftdev.Read(ch3, RxLength, ref readLength);                                  // Read Data CH1

                //-------------------------------------------------------------------- CH1 RAM 読出コマンド ----------------------------------------
                for (adrs = 0x1000; adrs < 0x1200; adrs++)                                  // RAM read 0x000 to 0x200 (0 to 511) CH1
                {
                    wbuf = FT232(adrs);                                                     // Get Command
                    ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                      // Write Command
                }

                //-------------------------------------------------------------------- FTDI バッファが一杯になったら読み出す -----------------------
                int tout = 0;
                do
                {
                    ftdev.GetRxBytesAvailable(ref RxLength);                                // Wait for return data
                    tout++;                                                                 // Timeout count                                                        
                } while (RxLength < 1023 & tout < 1023);                                    // Length = RAM data x2

                ftdev.Read(ch1, RxLength, ref readLength);                                  // Read Data CH1

                //-------------------------------------------------------------------- CH2 RAM 読出コマンド ----------------------------------------
                for (adrs = 0x1800; adrs < 0x1A00; adrs++)                                  // RAM read 0x000 to 0x200 (0 to 511) CH2
                {
                    wbuf = FT232(adrs);                                                     // Get Command
                    ftdev.Write(wbuf, wbuf.Length, ref writtenLength);                      // Write Command
                }

                //-------------------------------------------------------------------- FTDI バッファが一杯になったら読み出す -----------------------
                tout = 0;
                do
                {
                    ftdev.GetRxBytesAvailable(ref RxLength);                                // Wait for return data
                    tout++;                                                                 // Timeout count                                                        
                } while (RxLength < 1023 & tout < 1023);                                    // Length = RAM data x2

                ftdev.Read(ch2, RxLength, ref readLength);                                  // Read Data CH1

                //-------------------------------------------------------------------- 画面を初期化 ------------------------------------------------
                pictureBox1.Image = null;                                                   // Picture Box Clear

                DrawArea = new Bitmap(pw1, ph1);                                            // Picture Box Defined
                pictureBox1.Image = DrawArea;

                g1 = Graphics.FromImage(DrawArea);                                          // Make Graphics Object

                for (float y = phb1; y <= ph1 - phs1; y = y + phs1)                         // Draw Horizon Lines
                {
                    g1.DrawLine(gpen, 0, y, pw1, y);
                }

                for (int w = pws1; w <= pw1 - pws1; w = w + pws1)                           // Draw Vertical Lines
                {
                    g1.DrawLine(gpen, w, 0, w, ph1);
                }

                //-------------------------------------------------------------------- 画面を初期化 ------------------------------------------------
                pictureBox2.Image = null;                                                   // Picture Box Clear

                DrawArea = new Bitmap(pw2, ph2);                                            // Picture Box Defined
                pictureBox2.Image = DrawArea;

                g2 = Graphics.FromImage(DrawArea);                                          // Make Graphics Object

                for (int y = phs2; y <= ph2 - phs2; y = y + phs2)                           // Draw Horizon Lines
                {
                    g2.DrawLine(gpen, 0, y, pw2, y);
                }

                for (int w = pws2; w <= pw2 - pws2; w = w + pws2)                           // Draw Vertical Lines
                {
                    g2.DrawLine(gpen, w, 0, w, ph2);
                }

                //-------------------------------------------------------------------- 2 BYTE を INT に変換 ----------------------------------------
                for (int n = 0; n < 512; n++)                                               
                {
                    int w1 = 0;
                    int w2 = 0;

                    v1[n] = ((ch1[2 * n + 0] & 0x0F) * 0x0100 + ch1[2 * n + 1]);
                    v2[n] = ((ch2[2 * n + 0] & 0x0F) * 0x0100 + ch2[2 * n + 1]);

                    //------------------------------- 12bitの2の補数を整数に変換
                    if (v1[n] < 2048)
                    {
                        w1 = v1[n];                         // 0 to 2047
                    }
                    else
                    {
                        w1 = v1[n] - 4096;                  // -2047 to -1
                    }

                    //------------------------------- 12bitの2の補数を整数に変換
                    if (v2[n] < 2048)
                    {
                        w2 = v2[n];                         // 0 to 2047
                    }
                    else
                    {
                        w2 = v2[n] - 4096;                  // -2047 to -1
                    }

                    v1[n] = w1;                             // CH1 -2047 to 2047
                    v2[n] = w2;                             // CH2 -2047 to 2047

                    //------------------------------- データを最大値±1のDoubleに変換
                    Global.d1[n] = (double)((float)w1 / 2047);      // CH1 -1 to 1
                    Global.d2[n] = (double)((float)w2 / 2047);      // CH2 -1 to 1
                    d1[n] = (double)((float)w1 / 2047);             // CH1 -1 to 1
                    d2[n] = (double)((float)w2 / 2047);             // CH2 -1 to 1
                }

                //-------------------------------------------------------------------- 複素 FFT 処理 -----------------------------------------------
                FFT(d1, d2, 512, winsw);                                                    // FFT Size=512 Window=0:Hann, 1:Blackman, 2:Square

                //-------------------------------------------------------------------- 複素 FFT dB 変換と並び替え ----------------------------------
                for (int n = 0; n < 512; n++)                                
                {
                    double vr = System.Math.Sqrt(d1[n] * d1[n] + d2[n] * d2[n]);            // RMS calc

                    Global.ave[n] = 0.9 * Global.ave[n] + 0.1 * vr;                         // Averaging 10

                    if (n <= 255)                                                           // Swap and Log10
                    {
                        Global.ff[n + 256] = (double)(System.Math.Log10(Global.ave[n] * 2) * 20);       // Amp = +/-2047 Sin wave = 0dB
                    }
                    else
                    {
                        Global.ff[n - 256] = (double)(System.Math.Log10(Global.ave[n] * 2) * 20);       // Amp = +/-2047 Sin wave = 0dB
                    }
                }

                //-------------------------------------------------------------------- 波形描画 CH1 ------------------------------------------------
                for (int n = 0; n < 512; n++)                                          
                {
                    float vr = phh1 * 256 / amp1;
                    float hr = 32 / hrz;
                    float sx = (n - 2) * hr;
                    float sy = phh1 - vr * lv - phs1 * pos1;
                    float ex = (n - 1) * hr;
                    float ey = phh1 - vr * (float)Global.d1[n] - phs1 * pos1;
                    if (n > 1) g1.DrawLine(rpen, sx, sy, ex, ey);
                    lv = (float)Global.d1[n];
                }

                //-------------------------------------------------------------------- 波形描画 CH2 ------------------------------------------------
                for (int n = 0; n < 512; n++)                                        
                {
                    float vr = phh1 * 256 / amp2;
                    float hr = 32 / hrz;
                    float sx = (n - 2) * hr;
                    float sy = phh1 - vr * lv - phs1 * pos2;
                    float ex = (n - 1) * hr;
                    float ey = phh1 - vr * (float)Global.d2[n] - phs1 * pos2;
                    if (n > 1) g1.DrawLine(bpen, sx, sy, ex, ey);
                    lv = (float)Global.d2[n];
                }

                //-------------------------------------------------------------------- 波形描画 CH3 ------------------------------------------------
                float rate = (float)pw2 / (float)span;
                for (int n = 0; n < 512; n++)
                {
                    float sx = rate * (n - 2 - (512 - span) / 2);
                    float ex = rate * (n - 1 - (512 - span) / 2);
                    float vf = (float)Global.ff[n] * phs2 - phs2 * 5;
                    float sy = phh2 - lv / sdiv + phs2 * sref / sdiv;
                    float ey = phh2 - vf / sdiv + phs2 * sref / sdiv;
                    if (n > 1) g2.DrawLine(apen, sx, sy, ex, ey);
                    lv = vf;
                }

                cnt++;

                //-------------------------------------------------------------------- 画面描画 ----------------------------------------------------
                pictureBox1.Refresh();                                               

            } while (cnt<1);    // FTDIオープンしたまま波形を取り込み続ける場合は、ここの値を1以上のループ回数にする

            ftdev.Close();      // Close Device

        }

        private void button1_Click(object sender, EventArgs e)
        {
            //  START Button
            GetCir();
            timer1.Enabled = true;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            //  STOP Button
            timer1.Enabled = false;
        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              窓関数付き複素FFT
        //------------------------------------------------------------------------------------------------------------------
        private void FFT(double[] real , double[] imag, int length, int sw)
        {
            int i, j, k, l, half;
            double m, h, g, q, p, phase;
            double a = 0;
            double b = 0;
            double swap;

            half = length / 2;

            //*********************** 波形読込と同時に窓関数を掛ける **********************

            for (i = 0; i < length; i++)
            {

                phase = 2 * System.Math.PI * (float)i / (float)length;                  // 0 to 2*pi

                if (sw == 0)
                {
                    a = 1.414 * real[i] * (0.50 - 0.50 * System.Math.Cos(phase));       //Hanning
                    b = 1.414 * imag[i] * (0.50 - 0.50 * System.Math.Cos(phase));       //Hanning
                }
                if (sw == 1)
                {
                    a = 1.669 * real[i] * (0.35875 - 0.48829 * System.Math.Cos(phase) + 0.14128 * System.Math.Cos(2 * phase) - 0.01168 * System.Math.Cos(3 * phase));  //Blackman-Harris
                    b = 1.669 * imag[i] * (0.35875 - 0.48829 * System.Math.Cos(phase) + 0.14128 * System.Math.Cos(2 * phase) - 0.01168 * System.Math.Cos(3 * phase));  //Blackman-Harris
                }
                if (sw == 2)
                {
                    a = 1.000 * real[i];
                    b = 1.000 * imag[i];
                }

                real[i] = a;
                imag[i] = b;
            }

            //******************************* FFTメイン **********************************
            m = System.Math.Round(System.Math.Log(length) / System.Math.Log(2));

            l = length;
            h = 1;

            for (g = 1; g <= m; g++)
            {
                l = l / 2;
                k = 0;
                for (q = 1; q <= h; q++)
                {
                    p = 0;
                    for (i = k; i <= l + k - 1; i++)
                    {
                        j = i + l;
                        a = real[i] - real[j];
                        b = imag[i] - imag[j];
                        real[i] = real[i] + real[j];
                        imag[i] = imag[i] + imag[j];
                        if (p == 0)
                        {
                            real[j] = a;
                            imag[j] = b;
                        }
                        else
                        {
                            real[j] = a * System.Math.Cos(2 * System.Math.PI * p / (float)length) + b * System.Math.Sin(2 * System.Math.PI * p / (float)length);
                            imag[j] = b * System.Math.Cos(2 * System.Math.PI * p / (float)length) - a * System.Math.Sin(2 * System.Math.PI * p / (float)length);
                        }
                        p = p + h;
                    }
                    k = k + l + l;
                }
                h = h + h;
            }

            j = length / 2;

            for (i = 1; i <= length - 1; i++)
            {
                k = length;
                if (j < i)
                {
                    swap = real[i];
                    real[i] = real[j];
                    real[j] = swap;
                    swap = imag[i];
                    imag[i] = imag[j];
                    imag[j] = swap;
                }

                do
                {
                    k = k / 2;
                    j = j - k;
                    if (k == 0 && j == 0) break;
                } while ((j + k) >= k);

                j = j + k + k;
            }
        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              マウスで掴む
        //------------------------------------------------------------------------------------------------------------------
        private void pictureBox3_MouseDown(object sender, MouseEventArgs e)
        {
            //　マウスを押した座標
            int mx = e.X;
            int my = e.Y;

            //　ブロックサイズ定義
            int bsize = 28;                                                             // Block Width
            int oline = 4;                                                              // Outside Line

            //　ブロックの位置出し
            int nx = (int)Math.Round((float)((mx - oline) / bsize));                    // Block Locate 0 to 19
            int ny = (int)Math.Round((float)((my - oline) / bsize));                    // Block Locate 0 to 8

            //　クリックした座標と一致するブロックをサーチ
            for (int n=1; n<57; n++)
            {
                if (nx == Global.blkx[n] & ny == Global.blky[n])
                {
                    Global.fblk = n;                                                    // Detect Block No 
                }
            }

            //　一致したブロックをロックオン
            if (Global.fblk > 0)
            {
                Global.mousesw = 1;                                                     // Locked on
                Block(mx, my);
                param();
            }
        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              マウスで掴んだまま移動
        //------------------------------------------------------------------------------------------------------------------
        private void pictureBox3_MouseMove(object sender, MouseEventArgs e)
        {
            int mx = e.X;
            int my = e.Y;

            if (Global.mousesw==1) Block(mx, my);
        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              マウスを離す
        //------------------------------------------------------------------------------------------------------------------
        private void pictureBox3_MouseUp(object sender, MouseEventArgs e)
        {
            int mx = e.X;
            int my = e.Y;

            Global.mousesw = 0;
        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              ブロックの描画
        //------------------------------------------------------------------------------------------------------------------
        private void Block(int mx, int my)
        {
            int pw3 = pictureBox3.Size.Width;                                           // Get Picture Box size
            int ph3 = pictureBox3.Size.Height;
            int pws3 = (int)((float)pw3 / 10);
            int phs3 = ph3 / 10;
            int phh3 = ph3 / 2;

            //　描画色の指定
            Pen bpen = new Pen(Color.Black, 1);
            Pen lpen = new Pen(Color.Blue, 1);
            Pen rpen = new Pen(Color.Red, 1);
            Pen gpen = new Pen(Color.DarkGreen, 1);
            Pen apen = new Pen(Color.Aqua, 1);

            //　画面を初期化 
            pictureBox3.Image = null;                                                   // Picture Box Clear
            DrawArea = new Bitmap(pw3, ph3);                                            // Picture Box Defined
            pictureBox3.Image = DrawArea;

            //　グラフィックス・オブジェクトの作成 3
            Graphics g3;
            g3 = Graphics.FromImage(DrawArea);                                          // Make Graphics Object

            //　ブロックサイズ定義
            int bsize = 28;                                                             // Block Width
            int csize = 18;                                                             // Circle Width
            int ssize = 9;                                                              // Circle Small
            int hsize = bsize / 2;                                                      // Block Half
            int coff = (bsize - csize) / 2;                                             // Circle Offset
            int soff = (bsize - ssize) / 2;                                             // Circle Offset
            int ctie = 4;                                                               // Circle Tie

            //　編集エリア（外枠）の描画
            int oline = 4;
            int endx = oline + bsize * 20;
            int endy = oline + bsize * 10;
            g3.DrawLine(bpen, oline, oline, endx, oline);
            g3.DrawLine(bpen, oline, oline, oline, endy);
            g3.DrawLine(bpen, endx, oline, endx, endy);
            g3.DrawLine(bpen, oline, endy, endx, endy);

            //　クリック点の位置出し
            Global.blkx[Global.fblk] = (int)Math.Round((float)((mx - oline) / bsize));  // Block Locate 0 to 21
            Global.blky[Global.fblk] = (int)Math.Round((float)((my - oline) / bsize));  // Block Locate 0 to 8
            if (Global.blkx[Global.fblk] <  0) Global.blkx[Global.fblk] =  0;           // Limit X=0
            if (Global.blkx[Global.fblk] > 19) Global.blkx[Global.fblk] = 19;           // Limit X=19
            if (Global.blky[Global.fblk] <  0) Global.blky[Global.fblk] =  0;           // Limit Y=0
            if (Global.blky[Global.fblk] >  9) Global.blky[Global.fblk] =  9;           // Limit Y=9

            for (int n = 1; n < 57; n++)
            {
                //　ブロックのドット単位座標
                int bx = oline + bsize * Global.blkx[n];                                                    // Block Grid X = 4 to 592
                int by = oline + bsize * Global.blky[n];                                                    // Block Grid Y = 4 to 228

                //　ブロックの外枠と塗りつぶし
                if (n != Global.fblk) g3.FillRectangle(Brushes.LightGray, bx, by, bsize, bsize);            // BOX Fill
                if (n == Global.fblk) g3.FillRectangle(Brushes.LightPink, bx, by, bsize, bsize);            // BOX Fill
                g3.DrawRectangle(bpen, bx, by, bsize, bsize);                                               // BOX Outline

                //　ブロック図の構成
                byte L1 = (byte)((Global.bv[n] & 0xC000) / 0x4000);                                         // Line 1 Left
                byte L2 = (byte)((Global.bv[n] & 0x3000) / 0x1000);                                         // Line 2 Right
                byte L3 = (byte)((Global.bv[n] & 0x0C00) / 0x0400);                                         // Line 3 Upper
                byte L4 = (byte)((Global.bv[n] & 0x0300) / 0x0100);                                         // Line 4 Lower
                byte SH = (byte)((Global.bv[n] & 0x00F0) / 0x0010);                                         // Block shape
                byte ST = (byte)((Global.bv[n] & 0x000F) / 0x0001);                                         // String

                //　ブロック内部の図形
                if (L1 == 0x01) g3.DrawLine(lpen, bx, by + hsize, bx + coff, by + hsize);                   // Wire L-D
                if (L2 == 0x01) g3.DrawLine(lpen, bx + csize + coff, by + hsize, bx + bsize, by + hsize);   // Wire D-R
                if (L3 == 0x01) g3.DrawLine(lpen, bx + hsize, by, bx + hsize, by + coff);                   // Wire U-D
                if (L4 == 0x01) g3.DrawLine(lpen, bx + hsize, by + csize + coff, bx + hsize, by + bsize);   // Wire D-L
                if (L1 == 0x02) g3.DrawLine(lpen, bx, by + hsize, bx + hsize, by + hsize);                  // Wire L-C
                if (L2 == 0x02) g3.DrawLine(lpen, bx + hsize, by + hsize, bx + bsize, by + hsize);          // Wire C-R
                if (L3 == 0x02) g3.DrawLine(lpen, bx + hsize, by, bx + hsize, by + hsize);                  // Wire U-C
                if (L4 == 0x02) g3.DrawLine(lpen, bx + hsize, by + hsize, bx + hsize, by + bsize);          // Wire C-L
                if (SH == 0x01) g3.DrawEllipse(lpen, bx + hsize - 2, by + hsize - 2, ctie, ctie);           // Circle Tie
                if (SH == 0x02) g3.DrawEllipse(lpen, bx + coff, by + coff, csize, csize);                   // Circle Large
                if (SH == 0x03) g3.DrawRectangle(lpen, bx + coff, by + coff, csize, csize);                 // Rectangle
                if (SH == 0x04) g3.FillEllipse(Brushes.Red, bx + soff, by + soff, ssize, ssize);            // Circle Samll
                if (SH == 0x05) g3.FillEllipse(Brushes.Blue, bx + soff, by + soff, ssize, ssize);           // Circle Samll

                //　文字関係の変数
                Font fnt = new Font("Arial", 12);
                int txoff = (int)Math.Round(coff * 1.2);
                int tyoff = (int)Math.Round(coff * 1.2);

                //　フォント大（1文字）
                if (ST <= 0x03)
                {
                    txoff = (int)Math.Round(coff * 1.2);                                                    // Text Offset X
                    tyoff = (int)Math.Round(coff * 1.2);                                                    // Text Offset Y
                    fnt = new Font("Arial", 12);                                                            // Font
                }

                //　フォント小（3文字）
                if (ST >= 0x04)
                {
                    txoff = (int)Math.Round(coff * 0.8);                                                    // Text Offset X
                    tyoff = (int)Math.Round(coff * 1.8);                                                    // Text Offset Y
                    fnt = new Font("Arial", 7);                                                             // Font
                }

                //　表示テキスト
                if (ST == 0x01) g3.DrawString("X", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);        // Text MIX
                if (ST == 0x02) g3.DrawString("+", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);        // Text ADD
                if (ST == 0x03) g3.DrawString("V", fnt, Brushes.Blue, bx + txoff, by + tyoff);              // Text Const
                if (ST == 0x04) g3.DrawString("Nco", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text NCO
                if (ST == 0x05) g3.DrawString("A/D", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text ADC
                if (ST == 0x06) g3.DrawString("D/A", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text DAC
                if (ST == 0x07) g3.DrawString("CIC", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text CIC
                if (ST == 0x08) g3.DrawString("X^2", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text Square
                if (ST == 0x09) g3.DrawString("Sqr", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text Square
                if (ST == 0x0A) g3.DrawString("Agc", fnt, Brushes.Blue, bx + txoff, by + tyoff);            // Text AGC
                if (ST == 0x0B) g3.DrawString("Awg", fnt, Brushes.DarkViolet, bx + txoff, by + tyoff);      // Text AWG
                if (ST == 0x0C) g3.DrawString("PW", fnt, Brushes.Blue, bx + txoff, by + tyoff);             // Text PWM
                if (ST == 0x0D) g3.DrawString("CH1", fnt, Brushes.Red, bx + txoff, by + tyoff);             // Text CH1
                if (ST == 0x0E) g3.DrawString("CH2", fnt, Brushes.DarkBlue, bx + txoff, by + tyoff);        // Text CH2
                if (ST == 0x0F) g3.DrawString("A/D", fnt, Brushes.Blue, bx + txoff, by + tyoff);            // Text A/D (1Msps)
            }


        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              パラメータリストの表示
        //------------------------------------------------------------------------------------------------------------------
        private void param()
        {
            // Clock
            int ck = 128000;
            if (Global.sbsw == 0)
                ck = 128000;
            else
                ck = 1000;

            // Block No
            int bn = Global.fblk;
            textBox1.Text = bn.ToString();

            //　アップダウンスイッチの更新
            if (bn >0 & bn < 22) numericUpDown4.Value = Global.param1[bn];

            //　パラメータリストの表示　1Msps ADC
            if (bn == 1)
            {
                textBox11.Text = "AF 1Msps A/D";
                textBox2.Text = "Fix 1MHz";                                 // Clock
                textBox3.Text = "Analog Input";                             // Port
                textBox4.Text = Global.param2[bn].ToString();
                textBox5.Text = "";
                textBox6.Text = "";
                textBox7.Text = "";
                textBox8.Text = "";
                textBox9.Text = "";
                textBox10.Text = "";
            }
            //　パラメータリストの表示　NCO
            else if (bn == 2 | bn == 3)
            {
                if (Global.sbsw == 0) textBox11.Text = "NCO 5Hz-32MHz";
                if (Global.sbsw == 1) textBox11.Text = "NCO 1kHz-2000kHz";
                if (Global.sbsw == 0) textBox2.Text = "Fix 64MHz";          // Clock
                if (Global.sbsw == 1) textBox2.Text = "Fix 1MHz";           // Clock
                textBox3.Text = "Freq (Hz)";                                // Freq
                textBox4.Text = Global.param2[bn].ToString();
                textBox5.Text = "Phase (deg)";                              // Phase
                textBox6.Text = Global.param3[bn].ToString();
                textBox7.Text = "Amp (%)";                                  // Amplitude
                textBox8.Text = Global.param4[bn].ToString();
                if (Global.sbsw == 0) textBox9.Text = "DC offset (%)";      // DC offset
                if (Global.sbsw == 0) textBox10.Text = Global.param5[bn].ToString();
            }
            //　パラメータリストの表示　CIC
            else if (bn == 6 | bn == 7)
            {
                textBox11.Text = "CIC N=4";
                double f = ck / Math.Pow(2, Global.param1[bn] - 1);
                textBox2.Text = f.ToString() + "kHz";                       // Clock Low
                textBox3.Text = "Clock Hi";                                 // Clock Hi
                textBox4.Text = Global.param2[bn].ToString();
                if (Global.sbsw == 0) textBox5.Text = "Shift (bit)";        // Shift
                if (Global.sbsw == 0) textBox6.Text = Global.param3[bn].ToString();
                textBox7.Text = "";
                textBox8.Text = "";
                textBox9.Text = "";
                textBox10.Text = "";
            }
            //　パラメータリストの表示　Const
            else if (bn == 12)
            {
                textBox11.Text = "Const Value";
                double f = ck / Math.Pow(2, Global.param1[bn] - 1);
                textBox2.Text = f.ToString() + "kHz";                       // Clock 
                textBox3.Text = "DC +/-2047";                               // Value
                int cv = int.Parse(Global.param2[bn].ToString());
                if (cv > 2047) cv = cv - 4095;                              // Unsigned to Signed
                textBox4.Text = cv.ToString();
                textBox5.Text = "";
                textBox6.Text = "";
                textBox7.Text = "";
                textBox8.Text = "";
                textBox9.Text = "";
                textBox10.Text = "";
            }
            //　パラメータリストの表示　ADC 64Msps
            else if (bn == 14 | bn == 15)
            {
                if (Global.sbsw == 0) textBox11.Text = "RX 64Msps A/D";
                if (Global.sbsw == 1) textBox11.Text = "Not Available";
                textBox2.Text = "Fix 64MHz";                                // Clock
                textBox3.Text = "";
                textBox4.Text = "";
                textBox5.Text = "";
                textBox6.Text = "";
                textBox7.Text = "";
                textBox8.Text = "";
                textBox9.Text = "";
                textBox10.Text = "";
            }
            //　パラメータリストの表示　DAC 64Msps
            else if (bn == 19)
            {
                if (Global.sbsw == 0) textBox11.Text = "TX 64Msps D/A";
                if (Global.sbsw == 1) textBox11.Text = "Not Available";
                textBox2.Text = "Fix 64MHz";                                // Clock
                textBox3.Text = "";
                textBox4.Text = "";
                textBox5.Text = "";
                textBox6.Text = "";
                textBox7.Text = "";
                textBox8.Text = "";
                textBox9.Text = "";
                textBox10.Text = "";
            }
            //　パラメータリストの表示　その他（パラメータのないデバイス）
            else
            {
                if ((Global.sbsw == 0) & (bn == 4)) textBox11.Text = "Mixer 3in 1out";
                if ((Global.sbsw == 1) & (bn == 4)) textBox11.Text = "Mixer 2in 1out";
                if ((Global.sbsw == 0) & (bn == 5)) textBox11.Text = "Mixer 3in 1out";
                if ((Global.sbsw == 1) & (bn == 5)) textBox11.Text = "Mixer 2in 1out";
                if (bn == 8) textBox11.Text = "Square";
                if (bn == 9) textBox11.Text = "Square";
                if (bn == 10) textBox11.Text = "ADD 2in 1out";
                if (bn == 11) textBox11.Text = "Square Root";
                if (bn == 13) textBox11.Text = "AGC";
                if ((Global.sbsw == 0) & (bn == 16)) textBox11.Text = "AWG";
                if ((Global.sbsw == 0) & (bn == 17)) textBox11.Text = "AWG";
                if ((Global.sbsw == 1) & (bn == 16)) textBox11.Text = "Not Available";
                if ((Global.sbsw == 1) & (bn == 17)) textBox11.Text = "Not Available";
                if (bn == 18) textBox11.Text = "PWM";
                if (bn == 20) textBox11.Text = "Probe CH1";
                if (bn == 21) textBox11.Text = "Probe CH2";
                if (bn >= 22 & bn <= 25) textBox11.Text = "Jumper";
                if (bn >25) textBox11.Text = "Wire";
                double f = ck / Math.Pow(2, Global.param1[bn] - 1);

                if (Global.param1[bn] == 0)
                {
                    textBox2.Text = "";
                }
                else
                {
                    if (bn == 19)
                    {
                        textBox2.Text = "Fix 64MHz";
                    }
                    else
                    {
                        textBox2.Text = f.ToString() + "kHz";                       // Clock 
                    }
                }

                if (bn >= 22)
                {
                    textBox3.Text = "Node";
                    textBox4.Text = Global.param2[bn].ToString();
                }
                else
                {
                    textBox3.Text = "";
                    textBox4.Text = "";
                }
                textBox5.Text = "";
                textBox6.Text = "";
                textBox7.Text = "";
                textBox8.Text = "";
                textBox9.Text = "";
                textBox10.Text = "";
            }

        }

        //------------------------------------------------------------------------------------------------------------------
        //                                              ブロック図から回路データを得る
        //------------------------------------------------------------------------------------------------------------------
        private void GetCir()
        {
            int focus = 0;
            int[] cir = new int[20];
            int[,] map = new int[20, 10];

            //　マップのクリア
            for (int y = 0; y < 10; y++)
            {
                for (int x = 0; x < 20; x++)
                {
                    map[x, y] = 0;
                    int bn = BlockNo(x, y);
                    if (bn >= 26) Global.param2[bn] = 0;
                }
            }

            //　各ブロック1～17の出力ポートをサーチする
            for (int n=1; n<=17; n++)      
            {
                //　検索元ブロックの座標を得る
                int ox = Global.blkx[n];
                int oy = Global.blky[n];
                int cx = ox;
                int cy = oy;

                //　検索先の座標を得る
                ox = ox + 1;

                //　検索先のブロック番号を得る
                focus = BlockNo(ox, oy);                                                // Get Foucus Block No

                //　検索先が接続されたブロックならマップに最初のマーキング
                if (focus > 0)
                {
                    int cb = Global.bv[BlockNo(cx, cy)];                                // Circuit Block
                    int pb = Global.bv[BlockNo(ox, oy)];                                // Port Block
                    int c1 = (cb & 0xC000) / 0x4000;                                    // Line 1 Left
                    int c2 = (cb & 0x3000) / 0x1000;                                    // Line 2 Right
                    int c3 = (cb & 0x0C00) / 0x0400;                                    // Line 3 Upper
                    int c4 = (cb & 0x0300) / 0x0100;                                    // Line 4 Lower
                    int p1 = (pb & 0xC000) / 0x4000;                                    // Line 1 Left
                    int p2 = (pb & 0x3000) / 0x1000;                                    // Line 2 Right
                    int p3 = (pb & 0x0C00) / 0x0400;                                    // Line 3 Upper
                    int p4 = (pb & 0x0300) / 0x0100;                                    // Line 4 Lower
                    if (    (cx == ox - 1 & cy == oy & c2 == 1 & p1 == 2) |             // Left
                            (cx == ox + 1 & cy == oy & c1 == 1 & p2 == 2) |             // Right
                            (cx == ox & cy == oy - 1 & c4 == 1 & p3 == 2) |             // Upper
                            (cx == ox & cy == oy + 1 & c3 == 1 & p4 == 2) )             // Lower
                    {
                        map[ox, oy] = n;
                        Global.param2[focus] = n;
                        //　ブロックがジャンパーの場合は反対側もマーキング
                        int rb = 0;
                        if (focus == 22) rb = 23;
                        if (focus == 23) rb = 22;
                        if (focus == 24) rb = 25;
                        if (focus == 25) rb = 24;
                        if (rb > 0)
                        {
                            map[Global.blkx[rb], Global.blky[rb]] = n;
                            Global.param2[rb] = n;
                        }
                    }
                }

                //　検索ループの深さを決める（20個先のブロックまで調べる）
                for (int depth = 1; depth < 20; depth++)
                {

                    //　検索先がワイヤブロックなら検索開始
                    if (focus >= 22)
                    {
                        //　ワイヤブロック全体を検索
                        for (int b = 22; b <= 56; b++)
                        {
                            int L1 = 0;
                            int L2 = 0;
                            int L3 = 0;
                            int L4 = 0;
                            //　検索ブロックの座標を得る
                            int sx = Global.blkx[b];                                        // Get Locate X 
                            int sy = Global.blky[b];                                        // Get Locate Y 
                            //　隣接ブロックのマーク値を得る
                            if (sx > 0) L1 = map[sx - 1, sy];                               // Check Left  
                            if (sx < 19) L2 = map[sx + 1, sy];                              // Check Right  
                            if (sy > 0) L3 = map[sx, sy - 1];                               // Check Upper
                            if (sy < 9) L4 = map[sx, sy + 1];                               // Check Lower 
                            //　検索ブロックの番号を得る
                            int sn = BlockNo(sx, sy);                                       // Get Serching Block No
                            ushort sv = Global.bv[sn];                                      // Get Serching Block Shape
                            //　検索ブロックのワイヤ情報を得る
                            int s1 = (sv & 0xC000) / 0x4000;                                // Line 1 Left
                            int s2 = (sv & 0x3000) / 0x1000;                                // Line 2 Right
                            int s3 = (sv & 0x0C00) / 0x0400;                                // Line 3 Upper
                            int s4 = (sv & 0x0300) / 0x0100;                                // Line 4 Lower
                            //　隣接ブロックがマーク有なら結線状態をチェック
                            if (L1 > 0)                                                     // Check Left
                            {
                                //　隣接ブロックの番号を得る
                                int an = BlockNo(sx - 1, sy);                               // Get Adjasent Block No
                                //　隣接ブロックの配線形状を得る
                                ushort av = Global.bv[an];                                  // Get Adjasent Block Shape
                                //　隣接ブロックのワイヤ情報を得る
                                int a1 = (av & 0xC000) / 0x4000;                            // Line 1 Left
                                int a2 = (av & 0x3000) / 0x1000;                            // Line 2 Right
                                int a3 = (av & 0x0C00) / 0x0400;                            // Line 3 Upper
                                int a4 = (av & 0x0300) / 0x0100;                            // Line 4 Lower
                                //　隣接ブロックのノード番号を得る
                                int node = Global.param2[an];
                                //　検索ブロックと隣接ブロックが接続され隣接ブロックがワイヤならマーキング
                                if (s1 == 2 & a2 == 2 & node == n)
                                {
                                    map[sx, sy] = node;
                                    Global.param2[b] = node;
                                    //　ブロックがジャンパーの場合は反対側もマーキング
                                    int rb = 0;
                                    if (b == 22) rb = 23;
                                    if (b == 23) rb = 22;
                                    if (b == 24) rb = 25;
                                    if (b == 25) rb = 24;
                                    if (rb > 0)
                                    {
                                        map[Global.blkx[rb], Global.blky[rb]] = node;
                                        Global.param2[rb] = node;
                                    }
                                }
                            }
                            //　隣接ブロックがマーク有なら結線状態をチェック
                            if (L2 > 0)                                                     // Check Left
                            {
                                //　隣接ブロックの番号を得る
                                int an = BlockNo(sx + 1, sy);                               // Get Adjasent Block No
                                //　隣接ブロックの配線形状を得る
                                ushort av = Global.bv[an];                                  // Get Adjasent Block Shape
                                //　隣接ブロックのワイヤ情報を得る
                                int a1 = (av & 0xC000) / 0x4000;                            // Line 1 Left
                                int a2 = (av & 0x3000) / 0x1000;                            // Line 2 Right
                                int a3 = (av & 0x0C00) / 0x0400;                            // Line 3 Upper
                                int a4 = (av & 0x0300) / 0x0100;                            // Line 4 Lower
                                //　隣接ブロックのノード番号を得る
                                int node = Global.param2[an];
                                //　検索ブロックと隣接ブロックが接続され隣接ブロックがワイヤならマーキング
                                if (s2 == 2 & a1 == 2 & node == n)
                                {
                                    map[sx, sy] = node;
                                    Global.param2[b] = node;
                                    //　ブロックがジャンパーの場合は反対側もマーキング
                                    int rb = 0;
                                    if (b == 22) rb = 23;
                                    if (b == 23) rb = 22;
                                    if (b == 24) rb = 25;
                                    if (b == 25) rb = 24;
                                    if (rb > 0)
                                    {
                                        map[Global.blkx[rb], Global.blky[rb]] = node;
                                        Global.param2[rb] = node;
                                    }
                                }
                            }
                            //　隣接ブロックがマーク有なら結線状態をチェック
                            if (L3 > 0)                                                     // Check Left
                            {
                                //　隣接ブロックの番号を得る
                                int an = BlockNo(sx, sy - 1);                               // Get Adjasent Block No
                                //　隣接ブロックの配線形状を得る
                                ushort av = Global.bv[an];                                  // Get Adjasent Block Shape
                                //　隣接ブロックのワイヤ情報を得る
                                int a1 = (av & 0xC000) / 0x4000;                            // Line 1 Left
                                int a2 = (av & 0x3000) / 0x1000;                            // Line 2 Right
                                int a3 = (av & 0x0C00) / 0x0400;                            // Line 3 Upper
                                int a4 = (av & 0x0300) / 0x0100;                            // Line 4 Lower
                                //　隣接ブロックのノード番号を得る
                                int node = Global.param2[an];
                                //　検索ブロックと隣接ブロックが接続され隣接ブロックがワイヤならマーキング
                                if (s3 == 2 & a4 == 2 & node == n)
                                {
                                    map[sx, sy] = node;
                                    Global.param2[b] = node;
                                    //　ブロックがジャンパーの場合は反対側もマーキング
                                    int rb = 0;
                                    if (b == 22) rb = 23;
                                    if (b == 23) rb = 22;
                                    if (b == 24) rb = 25;
                                    if (b == 25) rb = 24;
                                    if (rb > 0)
                                    {
                                        map[Global.blkx[rb], Global.blky[rb]] = node;
                                        Global.param2[rb] = node;
                                    }
                                }
                            }
                            //　隣接ブロックがマーク有なら結線状態をチェック
                            if (L4 > 0)                                                     // Check Left
                            {
                                //　隣接ブロックの番号を得る
                                int an = BlockNo(sx, sy + 1);                               // Get Adjasent Block No
                                //　隣接ブロックの配線形状を得る
                                ushort av = Global.bv[an];                                  // Get Adjasent Block Shape
                                //　隣接ブロックのワイヤ情報を得る
                                int a1 = (av & 0xC000) / 0x4000;                            // Line 1 Left
                                int a2 = (av & 0x3000) / 0x1000;                            // Line 2 Right
                                int a3 = (av & 0x0C00) / 0x0400;                            // Line 3 Upper
                                int a4 = (av & 0x0300) / 0x0100;                            // Line 4 Lower
                                //　隣接ブロックのノード番号を得る
                                int node = Global.param2[an];
                                //　検索ブロックと隣接ブロックが接続され隣接ブロックがワイヤならマーキング
                                if (s4 == 2 & a3 == 2 & node == n)
                                {
                                    map[sx, sy] = node;
                                    Global.param2[b] = node;
                                    //　ブロックがジャンパーの場合は反対側もマーキング
                                    int rb = 0;
                                    if (b == 22) rb = 23;
                                    if (b == 23) rb = 22;
                                    if (b == 24) rb = 25;
                                    if (b == 25) rb = 24;
                                    if (rb > 0)
                                    {
                                        map[Global.blkx[rb], Global.blky[rb]] = node;
                                        Global.param2[rb] = node;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            //------------------------------------------ 回路データをブロック図から生成 ------------------------------------------
            //　回路のサイズ
            int size = 0;
            if (Global.sbsw == 0)
                size = 18;
            else
                size = 17;

            for (int n = 1; n <= size; n++)
            {
                int ob = 0;

                //　検索元ブロックの番号を得る
                if (Global.sbsw == 0)
                {                               // SDR Block HF
                    if (n == 1) ob = 4;         // MIX1 in U
                    if (n == 2) ob = 4;         // MIX1 in D
                    if (n == 3) ob = 4;         // MIX1 in L
                    if (n == 4) ob = 5;         // MIX2 in U
                    if (n == 5) ob = 5;         // MIX2 in D
                    if (n == 6) ob = 5;         // MIX2 in L
                    if (n == 7) ob = 6;         // CIC1
                    if (n == 8) ob = 7;         // CIC2
                    if (n == 9) ob = 8;         // SQR1
                    if (n == 10) ob = 9;        // SQR2
                    if (n == 11) ob = 10;       // ADD in U
                    if (n == 12) ob = 10;       // ADD in D
                    if (n == 13) ob = 11;       // SQRT
                    if (n == 14) ob = 18;       // PWM
                    if (n == 15) ob = 20;       // Probe CH1
                    if (n == 16) ob = 21;       // Probe CH2
                    if (n == 17) ob = 13;       // AGC
                    if (n == 18) ob = 19;       // DAC
                }
                else
                {                               // SDR Block AM
                    if (n == 1) ob = 4;         // MIX1 in U
                    if (n == 2) ob = 4;         // MIX1 in D
                    if (n == 3) ob = 5;         // MIX2 in U
                    if (n == 4) ob = 5;         // MIX2 in D
                    if (n == 5) ob = 6;         // CIC1 in
                    if (n == 6) ob = 6;         // CIC1 gain
                    if (n == 7) ob = 7;         // CIC2 in
                    if (n == 8) ob = 7;         // CIC2 gain
                    if (n == 9) ob = 8;         // SQR1
                    if (n == 10) ob = 9;        // SQR2
                    if (n == 11) ob = 10;       // ADD in U
                    if (n == 12) ob = 10;       // ADD in D
                    if (n == 13) ob = 11;       // SQRT
                    if (n == 14) ob = 18;       // PWM
                    if (n == 15) ob = 20;       // Probe CH1
                    if (n == 16) ob = 21;       // Probe CH2
                    if (n == 17) ob = 13;       // AGC
                    if (n == 18) ob = 19;       // DAC
                }

                //　検索元ブロックの座標を得る
                int ox = Global.blkx[ob];
                int oy = Global.blky[ob];

                //　検索先の座標を得る
                if (Global.sbsw == 0)
                {                               // SDR Block HF
                    if (n == 1) oy--;           // MIX1 in U
                    if (n == 2) oy++;           // MIX1 in D
                    if (n == 3) ox--;           // MIX1 in L
                    if (n == 4) oy--;           // MIX2 in U
                    if (n == 5) oy++;           // MIX2 in D
                    if (n == 6) ox--;           // MIX2 in L
                    if (n == 7) ox--;           // CIC1
                    if (n == 8) ox--;           // CIC2
                    if (n == 9) ox--;           // SQR1
                    if (n == 10) ox--;          // SQR2
                    if (n == 11) oy--;          // ADD in U
                    if (n == 12) oy++;          // ADD in D
                    if (n == 13) ox--;          // SQRT
                    if (n == 14) ox--;          // PWM
                    if (n == 15) ox--;          // Probe CH1
                    if (n == 16) ox--;          // Probe CH2
                    if (n == 17) ox--;          // AGC
                    if (n == 18) ox--;          // DAC
                }
                else
                {                               // SDR Block AM
                    if (n == 1) oy--;           // MIX1 in U
                    if (n == 2) oy++;           // MIX1 in D
                    if (n == 3) oy--;           // MIX2 in U
                    if (n == 4) oy++;           // MIX2 in D
                    if (n == 5) ox--;           // CIC1 in
                    if (n == 6) oy++;           // CIC1 gain
                    if (n == 7) ox--;           // CIC2 in
                    if (n == 8) oy--;           // CIC2 gain
                    if (n == 9) ox--;           // SQR1
                    if (n == 10) ox--;          // SQR2
                    if (n == 11) oy--;          // ADD in U
                    if (n == 12) oy++;          // ADD in D
                    if (n == 13) ox--;          // SQRT
                    if (n == 14) ox--;          // PWM
                    if (n == 15) ox--;          // Probe CH1
                    if (n == 16) ox--;          // Probe CH2
                    if (n == 17) ox--;          // AGC
                    if (n == 18) ox--;          // DAC
                }

                //　接続先のブロック番号を得る
                int bn = BlockNo(ox, oy);
                int node = 12;

                //　接続先のノード番号を得る
                if (bn >= 22)                       // Wire Blocks
                    node = Global.param2[bn];
                else if (bn >=1 & bn <=17)          // Circuit Blocks
                    node = bn;

                //　未定義ノードはConst(12)へ接続
                if (node == 0) node = 12;

                //　回路データを作成
                cir[n] = node;
            }

            //　回路データの表示
            String res = "";
            res = "Data sent "
                + cir[1].ToString() + ","
                + cir[2].ToString() + ","
                + cir[3].ToString() + ","
                + cir[4].ToString() + ","
                + cir[5].ToString() + ","
                + cir[6].ToString() + ","
                + cir[7].ToString() + ","
                + cir[8].ToString() + ","
                + cir[9].ToString() + ","
                + cir[10].ToString() + ","
                + cir[11].ToString() + ","
                + cir[12].ToString() + ","
                + cir[13].ToString() + ","
                + cir[14].ToString() + ","
                + cir[15].ToString() + ","
                + cir[16].ToString() + ","
                + cir[17].ToString() + ","
                + cir[18].ToString();

            textBox15.Text = res;

            //------------------------------------------ 回路データをUSBでボードへ伝送 -----------------------------------------
            //　USBポートオープン
            FTDI ftdev = new FTDI();                                                // Device Driver
            ftdev.OpenByIndex(0);                                                   // Open Device Index=0
            ftdev.SetBitMode(0, 2);                                                 // Mask=0, Mode=2(MPSSE)
            byte[] ret;
            uint writtenLength = 0;
            int ph = 0;
            ushort ph1 = 0;
            ushort ph2 = 0;
            ushort ph3 = 0;

            //　回路データをボードへ伝送
            for (int n = 1; n <= size; n++)
            {
                ushort comm = (ushort)(0x4000 + 32 * n + cir[n]);
                ret = FT232(comm);                                                  // Capture Start
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                Thread.Sleep(1);                                                    // Wait 1/16kHz (1ms)
            }

            //　クロックデータをボードへ伝送
            for (int n = 1; n <= 12; n++)
            {
                int cn = 0;
                if (n ==  1) cn = Global.param1[4];                                 // MIX1
                if (n ==  2) cn = Global.param1[5];                                 // MIX2
                if (n ==  3) cn = Global.param2[6];                                 // CIC1 HI
                if (n ==  4) cn = Global.param1[6];                                 // CIC1 LO
                if (n ==  5) cn = Global.param2[7];                                 // CIC2 HI
                if (n ==  6) cn = Global.param1[7];                                 // CIC2 LO
                if (n ==  7) cn = Global.param1[8];                                 // SQ1
                if (n ==  8) cn = Global.param1[9];                                 // SQ2
                if (n ==  9) cn = Global.param1[11];                                // SQRT
                if (n == 10) cn = Global.param1[10];                                // ADD
                if (n == 11) cn = Global.param1[13];                                // AGC
                if (n == 12) cn = Global.param1[16];                                // AWG
                ushort comm = (ushort)(0x5000 + 32 * n + cn);
                ret = FT232(comm);                                                  // Capture Start
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                Thread.Sleep(1);                                                    // Wait 1/16kHz (1ms)
            }

            //　NCO周波数パラメータをボードへ伝送
            int freq = Global.param2[2];
            if (Global.sbsw == 0)
            {                                                                       // SDR Block HF
                ph = (int)Math.Round(freq * 0.205887);
                ph1 = (ushort)((ph & 0xFF0000) / 0x10000);
                ph2 = (ushort)((ph & 0x00FF00) / 0x00100);
                ph3 = (ushort)((ph & 0x0000FF) / 0x00001);
                ret = FT232((ushort)(0x2000 + 2 * 0x100 + ph1));                    // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x2000 + 1 * 0x100 + ph2));                    // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x2000 + 0 * 0x100 + ph3));                    // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }
            else
            {                                                                       // SDR Block AM
                ph1 = (ushort)(freq / 1000);
                ret = FT232((ushort)(0x2000 + ph1));                                // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　NCOの周波数パラメータをボードへ伝送
            freq = Global.param2[3];
            if (Global.sbsw == 0)
            {                                                                       // SDR Block HF
                ph = (int)Math.Round(freq * 0.205887);
                ph1 = (ushort)((ph & 0xFF0000) / 0x10000);
                ph2 = (ushort)((ph & 0x00FF00) / 0x00100);
                ph3 = (ushort)((ph & 0x0000FF) / 0x00001);
                ret = FT232((ushort)(0x2800 + 2 * 0x100 + ph1));                    // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x2800 + 1 * 0x100 + ph2));                    // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x2800 + 0 * 0x100 + ph3));                    // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }
            else
            {                                                                       // SDR Block AM
                ph1 = (ushort)(freq / 1000);
                ret = FT232((ushort)(0x2000 + ph1));                                // Generate Freq Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　NCOの位相パラメータをボードに伝送
            ph1 = (ushort)(Math.Round(4.4666 * Global.param3[2]));
            ph2 = (ushort)(Math.Round(4.4666 * Global.param3[3]));
            ret = FT232((ushort)(0x6000 + ph1));                                // Generate Data
            ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            ret = FT232((ushort)(0x6800 + ph2));                                // Generate Data
            ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command

            //　NCOの振幅パラメータをボードに伝送
            if (Global.sbsw == 0)
            {                                                                       // SDR Block HF
                int amp1 = (ushort)(255 * Global.param4[2] / 100);
                int amp2 = (ushort)(255 * Global.param4[3] / 100);
                ret = FT232((ushort)(0x7000 + amp1));                               // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x7800 + amp2));                               // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }
            else
            {                                                                       // SDR Block AM
                int amp1 = (ushort)(1023 * Global.param4[2] / 100);
                int amp2 = (ushort)(1023 * Global.param4[3] / 100);
                ret = FT232((ushort)(0x7000 + amp1));                               // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x7800 + amp2));                               // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　NCOのDCパラメータをボードに伝送
            if (Global.sbsw == 0)
            {                                                                       // SDR Block HF
                int dc1 = (ushort)(511 * Global.param5[2] / 100);
                int dc2 = (ushort)(511 * Global.param5[3] / 100);
                ret = FT232((ushort)(0x7400 + dc1));                                // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232((ushort)(0x7C00 + dc2));                                // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　CICパラメータをボードに伝送
            if (Global.sbsw == 0)
            {                                                                       // SDR Block HF
                int cic = (ushort)(64 * Global.param3[6] + Global.param3[7]);
                ret = FT232((ushort)(0x8000 + cic));                                // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　CONSTパラメータをボードに伝送
            int con = (ushort)Global.param2[12];
            ret = FT232((ushort)(0x3000 + con));                                // Generate Data
            ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command

            //　NCOのリセット
            if (Global.sbsw == 0)
            {                                                                       // SDR Block HF
                ret = FT232(0x2600);                                                // Reset ON
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                ret = FT232(0x2700);                                                // Reset OFF
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　VACPパラメータのセット
            if (Global.sbsw == 1)
            {                                                                       // SDR Block AM
                int vcap = 523;
                ret = FT232((ushort)(0x8000 + vcap));                               // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
            }

            //　USBポートクローズ
            ftdev.Close();                                                      // Close Device
        }

        //　座標に存在するブロック番号を得る
        static int BlockNo(int fx, int fy)
        {
            int x = 0;
            int y = 0;
            int fb = 0;

            for (int b=1; b<=56; b++)
            {
                x = Global.blkx[b];
                y = Global.blky[b];

                if (x == fx & y == fy) fb = b;

            }

            return fb;
        }

        private void dimblock()
        {
            int n = 0;
            int nfreq = 0;
            int hck = 0;
            int lck = 0;

            if (Global.sbsw == 0)
            {                               // SDR Block HF
                nfreq = 810000;
                hck = 2;
                lck = 14;
            }
            else
            {                               // SDR Block AM
                nfreq = 810000;
                hck = 1;
                lck = 7;
            }

            Global.param1[1] = 8;           // ADC Clock
            Global.param2[1] = 0;           // ADC CH
            Global.param1[2] = hck;         // NCO Clock
            Global.param2[2] = nfreq;       // NCO Freq
            Global.param3[2] = 0;           // NCO Phase
            Global.param4[2] = 10;          // NCO Amp
            Global.param5[2] = 0;           // NCO DC
            Global.param1[3] = hck;         // NCO Clock
            Global.param2[3] = nfreq;       // NCO Freq
            Global.param3[3] = 90;          // NCO Phase
            Global.param4[3] = 10;          // NCO Amp
            Global.param5[3] = 0;           // NCO DC
            Global.param1[4] = hck;         // MIX Clock
            Global.param1[5] = hck;         // MIX Clock
            Global.param1[6] = lck;         // CIC Clock Low
            Global.param2[6] = hck;         // CIC Clock Hi
            Global.param3[6] = 44;          // CIC Shift
            Global.param1[7] = lck;         // CIC Clock Low
            Global.param2[7] = hck;         // CIC Clock Hi
            Global.param3[7] = 44;          // CIC Shift
            Global.param1[8] = lck;         // SQR Clock
            Global.param1[9] = lck;         // SQR Clock
            Global.param1[10] = lck;        // ADD Clock
            Global.param1[11] = lck;        // SQRT Clock
            Global.param1[12] = lck;        // Const Clock
            Global.param2[12] = 120;        // Const Value
            Global.param1[13] = lck;        // AGC Clock
            Global.param1[14] = hck;        // ADC Clock
            Global.param1[15] = hck;        // ADC Clock
            Global.param1[16] = hck;        // AWG Clock
            Global.param1[17] = hck;        // AWG Clock
            Global.param1[18] = lck;        // AGC Clock
            Global.param1[19] = hck;        // DAC Clock
            Global.param1[20] = lck;        // CH1 Clock
            Global.param1[21] = lck;        // CH2 Clock

            n++;
            Global.bv[n] = 0x103F;        // ADC 1Msps
            Global.blkx[n] = 0;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x1024;        // NCO 1
            Global.blkx[n] = 1;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x1024;        // NCO 2
            Global.blkx[n] = 2;
            Global.blky[n] = 8;
            n++;
            if (Global.sbsw == 0)
                Global.bv[n] = 0x5521;    // MIX 1 for HF
            else
                Global.bv[n] = 0x1521;    // MIX 1 for AM
            Global.blkx[n] = 3;
            Global.blky[n] = 8;
            n++;
            if (Global.sbsw == 0)
                Global.bv[n] = 0x5521;    // MIX 2 for HF
            else
                Global.bv[n] = 0x1521;    // MIX 2 for AM
            Global.blkx[n] = 4;
            Global.blky[n] = 8;
            n++;
            if (Global.sbsw == 0)
                Global.bv[n] = 0x5037;    // CIC 1 for HF
            else
                Global.bv[n] = 0x5137;    // CIC 1 for AM
            Global.blkx[n] = 5;
            Global.blky[n] = 8;
            n++;
            if (Global.sbsw == 0)
                Global.bv[n] = 0x5037;    // CIC 2 for HF
            else
                Global.bv[n] = 0x5437;    // CIC 2 for AM
            Global.blkx[n] = 6;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x5038;        // SQ 1
            Global.blkx[n] = 7;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x5038;        // SQ 2
            Global.blkx[n] = 8;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x1522;        // ADD
            Global.blkx[n] = 9;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x5039;        // SQR
            Global.blkx[n] = 10;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x1023;        // Const
            Global.blkx[n] = 11;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x503A;        // AGC
            Global.blkx[n] = 12;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x1035;        // ADC 1
            Global.blkx[n] = 13;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x1035;        // ADC 2
            Global.blkx[n] = 14;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x103B;        // AWG 1
            Global.blkx[n] = 15;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x103B;        // AWG 2
            Global.blkx[n] = 16;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x403C;        // PWM
            Global.blkx[n] = 17;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x4036;        // DAC
            Global.blkx[n] = 18;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x400D;        // Probe CH1
            Global.blkx[n] = 19;
            Global.blky[n] = 7;
            n++;
            Global.bv[n] = 0x400E;        // Probe CH2
            Global.blkx[n] = 19;
            Global.blky[n] = 8;
            n++;
            Global.bv[n] = 0x8040;        // Jumper in1
            Global.blkx[n] = 18;
            Global.blky[n] = 4;
            n++;
            Global.bv[n] = 0x2040;        // Jumper out1
            Global.blkx[n] = 18;
            Global.blky[n] = 5;
            n++;
            Global.bv[n] = 0x8050;        // Jumper in2
            Global.blkx[n] = 18;
            Global.blky[n] = 6;
            n++;
            Global.bv[n] = 0x2050;        // Jumper out2
            Global.blkx[n] = 18;
            Global.blky[n] = 7;
            n++;
            Global.bv[n] = 0x8800;        // Line L-U
            Global.blkx[n] = 0;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x8800;        // Line L-U
            Global.blkx[n] = 1;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x8200;        // Line L-D
            Global.blkx[n] = 2;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x8200;        // Line L-D
            Global.blkx[n] = 3;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x2800;        // Line U-R
            Global.blkx[n] = 4;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x2800;        // Line U-R
            Global.blkx[n] = 5;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x2200;        // Line D-R
            Global.blkx[n] = 6;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x2200;        // Line D-R
            Global.blkx[n] = 7;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x8A10;        // Line L-U-D
            Global.blkx[n] = 8;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x8A10;        // Line L-U-D
            Global.blkx[n] = 9;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA810;        // Line L-U-R
            Global.blkx[n] = 10;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA810;        // Line L-U-R
            Global.blkx[n] = 11;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA210;        // Line L-D-R
            Global.blkx[n] = 12;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA210;        // Line L-D-R
            Global.blkx[n] = 13;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA000;        // Line L-R
            Global.blkx[n] = 14;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA000;        // Line L-R
            Global.blkx[n] = 15;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA000;        // Line L-R
            Global.blkx[n] = 16;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA000;        // Line L-R
            Global.blkx[n] = 17;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA000;        // Line L-R
            Global.blkx[n] = 18;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0xA000;        // Line L-R
            Global.blkx[n] = 19;
            Global.blky[n] = 9;
            n++;
            Global.bv[n] = 0x0A00;        // Line U-D
            Global.blkx[n] = 19;
            Global.blky[n] = 6;
            n++;
            Global.bv[n] = 0x0A00;        // Line U-D
            Global.blkx[n] = 19;
            Global.blky[n] = 5;
            n++;
            Global.bv[n] = 0x0A00;        // Line U-D
            Global.blkx[n] = 19;
            Global.blky[n] = 4;
            n++;
            Global.bv[n] = 0x0A00;        // Line U-D
            Global.blkx[n] = 19;
            Global.blky[n] = 3;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 19;
            Global.blky[n] = 2;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 19;
            Global.blky[n] = 1;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 19;
            Global.blky[n] = 0;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 18;
            Global.blky[n] = 3;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 18;
            Global.blky[n] = 2;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 18;
            Global.blky[n] = 1;
            n++;
            Global.bv[n] = 0xAA10;        // Line Cross Tie
            Global.blkx[n] = 18;
            Global.blky[n] = 0;
        }

        private void textBox4_TextChanged(object sender, EventArgs e)       // Parameter 1
        {
            int bn = Global.fblk;                                           // Get Block No
            int va = 0;
            if (textBox4.Text != "" & textBox4.Text != "-") va = int.Parse(textBox4.Text);         // Read text to int
            if (va < 0) va = 4095 + va;                                     // Signed to Unsigned
            Global.param2[bn] = va;                                         // Write New Value
            if ((bn == 2) | (bn == 3))                                      // NCO f1=f2 in SDR Block AM 
            {
                if (Global.sbsw == 1)
                {
                    Global.param2[2] = va;
                    Global.param2[3] = va;
                }
            }
        }

        private void textBox6_TextChanged(object sender, EventArgs e)       // Parameter 2
        {
            int bn = Global.fblk;                                           // Get Block No
            int va = 0;
            if (textBox6.Text != "") va = int.Parse(textBox6.Text);         // Read text to int
            Global.param3[bn] = va;                                         // Write New Value
        }

        private void textBox8_TextChanged(object sender, EventArgs e)       // Parameter 3
        {
            int bn = Global.fblk;                                           // Get Block No
            int va = 0;
            if (textBox8.Text != "") va = int.Parse(textBox8.Text);         // Read text to int
            Global.param4[bn] = va;                                         // Write New Value
        }

        private void textBox10_TextChanged(object sender, EventArgs e)      // Parameter 4
        {
            int bn = Global.fblk;                                           // Get Block No
            int va = 0;
            if (textBox10.Text != "") va = int.Parse(textBox10.Text);       // Read text to int
            Global.param5[bn] = va;                                         // Write New Value
        }

        private void numericUpDown4_ValueChanged(object sender, EventArgs e)
        {
            int ck = 128000;
            int bn = Global.fblk;                                           // Get Block No
            int clk = (int)numericUpDown4.Value;                            // Get New Clock

            if (Global.sbsw == 0)                                           // Clock 128MHz or 1MHz
                ck = 128000;
            else
                ck = 1000;

            double f = ck / Math.Pow(2, clk - 1);                           // Calc New clock freq
            textBox2.Text = f.ToString() + "kHz";                           // Display New Clock 

            Global.param1[bn] = clk;                                        // Write New Clock

            if (bn == 20 | bn ==21)                                         // Probe Clock CH1=CH2
            {
                Global.param1[20] = clk;
                Global.param1[21] = clk;
            }
        }

        private void hScrollBar3_Scroll(object sender, ScrollEventArgs e)
        {
            //　スクロールバーの値を取得
            int hmin = int.Parse(textBox12.Text);
            int hmax = int.Parse(textBox14.Text);
            int hval = hScrollBar3.Value;
            int val = (int)(hmin + ((float)hval / 1000.0) * (hmax - hmin));
            int freq = hmin + hval * (hmax - hmin) / 1000;
            textBox13.Text = freq.ToString();
            if ((radioButton1.Checked == true) | (radioButton2.Checked == true)) Global.param2[2] = freq;
            if ((radioButton1.Checked == true) | (radioButton3.Checked == true)) Global.param2[3] = freq;
            if (radioButton4.Checked == true)
            {
                if (val < 0) val = 0;
                if (val > 2047) val = 2047;
                Global.param2[12] = val;
            }
            if (timer1.Enabled == true) GetCir();

        }

        private void aWGToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void aMRADIOToolStripMenuItem_Click(object sender, EventArgs e)
        {
            int[,] map = new int[20,10];

            //------------------------------------- 例題・AMラジオ ----------------------------------
            if (Global.sbsw == 0)   // SDR Block HF
            {
                map = new int[,] {  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0, 0, 2,28, 0,32,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0,33,40, 4, 6,36, 8,29, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0,14,34,23,35, 0, 0, 0,10,11,38,18, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0,30,41, 5, 7,39, 9,27, 0,54,13,42,22, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0, 0, 3,26, 0,31,21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,56 },
                                    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,55 },
                                    { 1,12,15,16,17,19,24,25,37,43,44,45,46,47,48,49,50,51,52,53 }  };

                int nfreq = 810000;
                int hck = 2;
                int lck = 14;

                Global.param1[2] = hck;         // NCO Clock
                Global.param2[2] = nfreq;       // NCO Freq
                Global.param3[2] = 0;           // NCO Phase
                Global.param4[2] = 10;          // NCO Amp
                Global.param5[2] = 0;           // NCO DC
                Global.param1[3] = hck;         // NCO Clock
                Global.param2[3] = nfreq;       // NCO Freq
                Global.param3[3] = 90;          // NCO Phase
                Global.param4[3] = 10;          // NCO Amp
                Global.param5[3] = 0;           // NCO DC
                Global.param1[4] = hck;         // MIX Clock
                Global.param1[5] = hck;         // MIX Clock
                Global.param1[6] = lck;         // CIC Clock Low
                Global.param2[6] = hck;         // CIC Clock Hi
                Global.param3[6] = 44;          // CIC Shift
                Global.param1[7] = lck;         // CIC Clock Low
                Global.param2[7] = hck;         // CIC Clock Hi
                Global.param3[7] = 44;          // CIC Shift
                Global.param1[8] = lck;         // SQR Clock
                Global.param1[9] = lck;         // SQR Clock
                Global.param1[10] = lck;        // ADD Clock
                Global.param1[11] = lck;        // SQRT Clock
                Global.param1[13] = lck;        // AGC Clock
                Global.param1[14] = hck;        // ADC Clock
                Global.param1[15] = hck;        // ADC Clock
                Global.param1[18] = lck;        // AGC Clock
                Global.param1[20] = lck;        // CH1 Clock
                Global.param1[21] = lck;        // CH2 Clock
            }
            else                    // SDR Block AM
            {
                map = new int[,] {  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 3,28, 0, 0,32,20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0, 5,40, 6,36, 8,29, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 1,35,23,34, 0, 0,10,11,39,18, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 0, 4,41, 7,38, 9,27, 0,31,13,42,22, 0, 0, 0, 0, 0, 0, 0 },
                                    { 0, 2,26, 0, 0,30,21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,33 },
                                    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,54 },
                                    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,56 },
                                    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,55 },
                                    {14,12,15,16,17,19,24,25,37,43,44,45,46,47,48,49,50,51,52,53 }  };

                int nfreq = 810000;
                int hck = 1;
                int lck = 7;

                Global.param1[1] = 1;           // ADC Clock
                Global.param2[1] = 0;           // ADC CH
                Global.param1[2] = hck;         // NCO Clock
                Global.param2[2] = nfreq;       // NCO Freq
                Global.param3[2] = 0;           // NCO Phase
                Global.param4[2] = 50;          // NCO Amp
                Global.param1[3] = hck;         // NCO Clock
                Global.param2[3] = nfreq;       // NCO Freq
                Global.param3[3] = 90;          // NCO Phase
                Global.param4[3] = 50;          // NCO Amp
                Global.param1[4] = hck;         // MIX Clock
                Global.param1[5] = hck;         // MIX Clock
                Global.param1[6] = lck;         // CIC Clock Low
                Global.param2[6] = hck;         // CIC Clock Hi
                Global.param1[7] = lck;         // CIC Clock Low
                Global.param2[7] = hck;         // CIC Clock Hi
                Global.param1[8] = lck;         // SQR Clock
                Global.param1[9] = lck;         // SQR Clock
                Global.param1[10] = lck;        // ADD Clock
                Global.param1[11] = lck;        // SQRT Clock
                Global.param1[13] = lck;        // AGC Clock
                Global.param1[18] = lck;        // AGC Clock
                Global.param1[20] = lck;        // CH1 Clock
                Global.param1[21] = lck;        // CH2 Clock
            }

            for (int x = 0; x < 20; x++)
            {
                for (int y = 0; y < 10; y++)
                {
                    if (map[y, x] != 0)
                    {
                        Global.blkx[map[y, x]] = x;
                        Global.blky[map[y, x]] = y;
                        Block(y, x);
                    }
                }
            }
        }

        private void textBox13_TextChanged(object sender, EventArgs e)
        {
            int freq = int.Parse(textBox13.Text);
            int hmin = int.Parse(textBox12.Text);
            int hmax = int.Parse(textBox14.Text);
            int hval = 1000 * (freq - hmin) / (hmax - hmin);
            if ((freq >= hmin) & (freq <= hmax))
            {
                hScrollBar3.Value = hval;
                //GetCir();
            }
        }

        private void sDRBlockHFToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Global.sbsw = 0;
            sDRBlockHFToolStripMenuItem.Checked = true;
            sDRBlockAMToolStripMenuItem.Checked = false;
            dimblock();
            Block(0, 0);
            numericUpDown4.Maximum = 17;
        }

        private void sDRBlockAMToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Global.sbsw = 1;
            sDRBlockHFToolStripMenuItem.Checked = false;
            sDRBlockAMToolStripMenuItem.Checked = true;
            dimblock();
            Block(0, 0);
            numericUpDown4.Maximum = 11;
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void sAVEToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //--------------------- ファイルセーブ -------------------------
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.FileName = "block_1.csv";
            sfd.InitialDirectory = @"C:\";
            sfd.Filter = "CSV File(*.csv)|*.csv|All Files(*.*)|*.*";
            sfd.FilterIndex = 2;
            sfd.Title = "Save File";
            //ダイアログボックスを閉じる前に現在のディレクトリを復元するようにする
            sfd.RestoreDirectory = true;
            //既に存在するファイル名を指定したとき警告する
            sfd.OverwritePrompt = true;
            //存在しないパスが指定されたとき警告を表示する
            sfd.CheckPathExists = true;
            //ダイアログを表示する
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                //　ブロック全部のXY座標をセーブ
                for (int n = 1; n <= 57; n++)
                {
                    //　1行をカンマ区切りで作成
                    string data =   n.ToString() + "," +
                                    Global.blkx[n].ToString() + "," +
                                    Global.blky[n].ToString() + "," +
                                    "0,0,0" +
                                    Environment.NewLine;
                    //　ファイルへ書き出し(1行目はオールクリア後)
                    if (n == 1) File.WriteAllText(sfd.FileName, data);
                    if (n >= 2) File.AppendAllText(sfd.FileName, data);
                }
                //　パラメータのセーブ
                for (int n = 1; n <= 21; n++)
                {
                    //　1行をカンマ区切りで作成
                    string data =   n.ToString() + "," +
                                    Global.param1[n].ToString() + "," +
                                    Global.param2[n].ToString() + "," +
                                    Global.param3[n].ToString() + "," +
                                    Global.param4[n].ToString() + "," +
                                    Global.param5[n].ToString() + 
                                    Environment.NewLine;
                    //　ファイルへ書き出し
                    File.AppendAllText(sfd.FileName, data);
                }
            }
        }

        private void lOADToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //--------------------- ファイルロード -------------------------
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.FileName = "default.csv";
            ofd.InitialDirectory = @"C:\";
            ofd.Filter = "CSV File(*.csv)|*.csv|All Files(*.*)|*.*";
            ofd.FilterIndex = 2;
            ofd.Title = "Load File";
            ofd.RestoreDirectory = true;
            //存在しないファイルの名前が指定されたとき警告を表示する
            ofd.CheckFileExists = true;
            //存在しないパスが指定されたとき警告を表示する
            ofd.CheckPathExists = true;
            //ダイアログを表示する
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                //　1行をカンマ区切り込みで読み出す
                string[] readText = File.ReadAllLines(ofd.FileName);
                //　ブロックXY座標の読出し
                for (int n = 0; n < 57; n++)
                {
                    //　カンマを分割して配列にする
                    string[] stv = readText[n].Split(',');
                    //　数値に変換して変数に格納
                    Global.blkx[n + 1] = int.Parse(stv[1]);
                    Global.blky[n + 1] = int.Parse(stv[2]);
                }
                //　パラメータの読出し
                for (int n = 57; n < 78; n++)
                {
                    //　カンマを分割して配列にする
                    string[] stv = readText[n].Split(',');
                    //　数値に変換して変数に格納
                    int bn = int.Parse(stv[0]);
                    Global.param1[bn] = int.Parse(stv[1]);
                    Global.param2[bn] = int.Parse(stv[2]);
                    Global.param3[bn] = int.Parse(stv[3]);
                    Global.param4[bn] = int.Parse(stv[4]);
                    Global.param5[bn] = int.Parse(stv[5]);
                }
                //　ブロック全部を描画
                Block(0,0);
            }
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            //　USBポートオープン
            FTDI ftdev = new FTDI();                                            // Device Driver
            ftdev.OpenByIndex(0);                                               // Open Device Index=0
            ftdev.SetBitMode(0, 2);                                             // Mask=0, Mode=2(MPSSE)
            byte[] ret;
            uint writtenLength = 0;
            //--------------------- AWG波形ロード -------------------------
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.FileName = "wave.csv";
            ofd.InitialDirectory = @"C:\";
            ofd.Filter = "CSV File(*.csv)|*.csv|All Files(*.*)|*.*";
            ofd.FilterIndex = 2;
            ofd.Title = "Load AWG Waveform File";
            ofd.RestoreDirectory = true;
            //存在しないファイルの名前が指定されたとき警告を表示する
            ofd.CheckFileExists = true;
            //存在しないパスが指定されたとき警告を表示する
            ofd.CheckPathExists = true;
            //ダイアログを表示する
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                //　1行をカンマ区切り込みで読み出す
                string[] readText = File.ReadAllLines(ofd.FileName);
                int leng = readText.Length;
                //　ブロックXY座標の読出し
                for (int n = 0; n < leng; n++)
                {
                    //　カンマを分割して配列にする
                    string[] stv = readText[n].Split(',');
                    //　数値に変換して変数に格納
                    Global.wave[n, 0] = float.Parse(stv[0]);
                    Global.wave[n, 1] = float.Parse(stv[1]);
                }
                //　データをFPGA内のRAMに転送
                for (int n = 0; n < leng; n++)
                {
                    int dat = (int)(Global.wave[n, 0] * 2047);
                    if (dat < 0) dat = dat + 4096;
                    ret = FT232((ushort)(0x9000 + dat));                                // Generate Data
                    ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                }
                // AWGのサイズをFPGAに転送して確定させる
                ret = FT232((ushort)(0xB000 + leng));                                   // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                        // Write Command
                //　データをFPGA内のRAMに転送
                for (int n = 0; n < leng; n++)
                {
                    int dat = (int)(Global.wave[n, 1] * 2047);
                    if (dat < 0) dat = dat + 4096;
                    ret = FT232((ushort)(0xA000 + dat));                                // Generate Data
                    ftdev.Write(ret, ret.Length, ref writtenLength);                    // Write Command
                }
                // AWGのサイズをFPGAに転送して確定させる
                ret = FT232((ushort)(0xB000 + leng));                                   // Generate Data
                ftdev.Write(ret, ret.Length, ref writtenLength);                        // Write Command
            }
            //　USBポートクローズ
            ftdev.Close();                                                              // Close Device
        }

        private void cLEARToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //　ブロック初期化
            dimblock();
            //　ブロック全部を描画
            Block(0, 0);
        }

        private void sTARTToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //  START Button
            GetCir();
            timer1.Enabled = true;
        }

        private void sTOPToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //  STOP Button
            timer1.Enabled = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void waveCapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //--------------------- ファイルセーブ -------------------------
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.FileName = "wavecap_1.csv";
            sfd.InitialDirectory = @"C:\";
            sfd.Filter = "CSV File(*.csv)|*.csv|All Files(*.*)|*.*";
            sfd.FilterIndex = 2;
            sfd.Title = "Wave Data Capture";
            //ダイアログボックスを閉じる前に現在のディレクトリを復元するようにする
            sfd.RestoreDirectory = true;
            //既に存在するファイル名を指定したとき警告する
            sfd.OverwritePrompt = true;
            //存在しないパスが指定されたとき警告を表示する
            sfd.CheckPathExists = true;
            //ダイアログを表示する
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                //　ブロック全部のXY座標をセーブ
                for (int n = 1; n <= 512; n++)
                {
                    //　1行をカンマ区切りで作成
                    string data =   n.ToString() + "," +
                                    Global.d1[n].ToString() + "," +
                                    Global.d2[n].ToString() + "," +
                                    Global.ff[n].ToString() + "," +
                                    Environment.NewLine;
                    //　ファイルへ書き出し(1行目はオールクリア後)
                    if (n == 1) File.WriteAllText(sfd.FileName, data);
                    if (n >= 2) File.AppendAllText(sfd.FileName, data);
                }
            }
        }
    }
}
